/*
 * RGB LED light rope reaction game
 * 
 * External devices connections to the Arduino:
 * 1) RGB LED light rope with 150 leds
 *       Light Rope     Arduino
 *          DI           pin 2
 *          VCC          5V
 *          GND          GND
 * 2) Push button
 *       Push Button    Arduino
 *          leg 1        pin 3
 *          leg 2        GND
 * 3) Buzzer 
 *          Buzzer       Arduino
 *          leg 1        pin 4
 *          leg 2        GND
 * 
 * 
 * Connection for programming the Pro Mini
 * Ignore this if using the Arduino Uno
 * ProMini  USB-TTL
 * DTR(GRN) DTR
 * Tx       Rx
 * Rx       Tx
 * Vcc      5V
 * GND      GND
 * 
 * 2018 c Enoch Hwang
 */

 //////////////////////////////////////////////////////////////
//// push button and speaker stuff
#define buttonPin 3
#define speaker 4
bool buttonPressed() {
  // active low
  if (digitalRead(buttonPin) == LOW) {
    return true;
  } else {
    return false;
  }
}


//////////////////////////////////////////////////////////////
//// RGB LED ring stuff
#include "WS2812.h"
#define DI_PIN 2      // Digital pin that the DI signal is connected to
#define LED_COUNT 150  // Number of LEDs in the ring or strip
const int TOP = 40;
const int BOTTOM = 0;
int score;

// Parameter 1 = number of pixels in leds,  neopixel stick has 8
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_RGB     Pixels are wired for RGB bitstream
//   NEO_GRB     Pixels are wired for GRB bitstream, correct for neopixel stick
//   NEO_GRBW    very different; interesting
//   NEO_KHZ400  400 KHz bitstream (e.g. FLORA pixels)
//   NEO_KHZ800  800 KHz bitstream (e.g. High Density LED leds), correct for neopixel stick

WS2812 rgb = WS2812(LED_COUNT, DI_PIN, NEO_GRB + NEO_KHZ800);

// blue color from LED_COUNT downto n
void blue(int n) {
  for (int i=rgb.numPixels(); i>=n; i--) {
    rgb.setPixelColor(i, 0,0,100);
  }
  rgb.show();
}

// display rainbow color from LED_COUNT downto TOP
void rainbowTop() {
  int color;
  for (int i=rgb.numPixels(); i>TOP; i--) {
    color = random(0, 256);
    rgb.setPixelColor(i, Wheel(color));
  }
  rgb.setPixelColor(TOP+1, 0,0,255);
  rgb.show();  
}

const int melody[] = {
  294, 330, 349, 392, 440, 587, 659, 698, 784, 988, 1047, 1175, 1319, 1397
};

// random flash color from TOP to LED_COUNT
void RGBrandom(int n) {
  int led, color;
  for (int j=0; j<n; j++) {
    for (int i=0; i<rgb.numPixels()/2; i++) { // for so many leds
      led = random(TOP+1, rgb.numPixels()+1);
      if (random(0, 3)) {
        color = random(0, 256);
        rgb.setPixelColor(led, Wheel(color));
      } else {
        rgb.setPixelColor(led, 0);
      }
    }
    rgb.show();
    //tone(speaker, 350, 10);
    tone(speaker, melody[random(0, 14)], 70);
    delay(70);
  }
  noTone(speaker);
  rainbowTop();
}


// blue led falls down from TOP to BOTTOM
// returns how far from BOTTOM for the score
int fall(int demoPressed=LED_COUNT) {
  int wait;
  if (score == 0) wait = random(8,11);  // adjust speed of fall for first time
  else if (score > 24) wait = random(3,6);  // adjust speed of fall for third time
  else wait = random(5,8);  // adjust speed of fall for second time
  Serial.print("wait=");
  Serial.println(wait);
  int i;
  for(i=TOP; i>=BOTTOM; i--) {
    rgb.setPixelColor(i-2, 0,0,255);  //Third Dot Color
    rgb.setPixelColor(i-1, 0,0,255);   //Second Dot Color
    rgb.setPixelColor(i, 0,0,100);     //Center Dot Color
    rgb.setPixelColor(i+1, 0,0,60);   //Second Dot Color
    rgb.setPixelColor(i+2, 0,0,20);  //Third Dot Color
    rgb.setPixelColor(i+3, 0,0,0); //Clears the dots after the 3rd color
    rgb.show();
    if (buttonPressed()) {
      break;
    }
    if ((i-2) == demoPressed) {
      break;
    }
    delay(wait);
  }
  return i;
}

void splashRed() {
  int led;
  for(int j=0; j<20; j++) {     // do it so many times
    for(int i=BOTTOM; i<rgb.numPixels(); i++) { // for rgb.numPixels()
      led = random(BOTTOM, rgb.numPixels());  // random led between BOTTOM and number of pixels
      switch(random(0,3)) {     // random intensity
      case 0:
        rgb.setPixelColor(led, 0,0,0);
        break;
      case 1:
        rgb.setPixelColor(led, 10,0,0);
        break;
      default:
        rgb.setPixelColor(led, 255,0,0);
        break;
      }
    }
    rgb.show();
    delay(50);
  }
}

// show score as green led bar going from 0 up to BOTTOM
void pushUpScore() {
  rgb.clear();
  rainbowTop();
  rgb.show();
  for(int i=0; i<score; i++) { // for BOTTOM number of leds
    rgb.setPixelColor(i, 0,255,0);
    rgb.show();
    delay(20);    
  }
}


void go(int demoPressed=LED_COUNT) {
  rgb.clear();
  rgb.show();
  int n=random(1,5)*10;
  RGBrandom(n);
  // go
  int s = fall(demoPressed);
  Serial.print("s=");
  Serial.print(s);
  Serial.print(" score=");
  Serial.print((TOP-s)/3);

    if (s >= 0) {
      tone(speaker, 800, 200);  // win tone
      score = score + ((TOP-s)/3) + 2;
      delay(1500);
    } else {
      tone(speaker,130,200);  // lost tone
      splashRed();
    }
    pushUpScore();
    Serial.print(" accumulated score=");
    Serial.println(score);
    delay(2000);
    noTone(speaker);
}

void song() {
  tone(speaker, 349);
  delay(500);
  tone(speaker, 262);
  delay(180);
  noTone(speaker);
  delay(40);
  tone(speaker, 262);
  delay(180);
  tone(speaker, 294);
  delay(400);
  tone(speaker, 262);
  delay(800);
  noTone(speaker);
  delay(150);
  tone(speaker, 330);
  delay(500);
  tone(speaker, 349);
  delay(800);
  noTone(speaker);
  pinMode(speaker,INPUT);  // need this to turn off noise  
}

void win() {
  song();  
  rainbowCycle(1);  
}

void lost() {
  for (int i=0; i<rgb.numPixels(); i++) {
    rgb.setPixelColor(i, 100,0,0);
  }
  rgb.show();
  tone(speaker,220,700);
  delay(700);
  tone(speaker,130,1000);
  delay(1000);
  noTone(speaker);
  delay(2000);
}

bool played;
unsigned long nextPlay;

void play() {
  score = 0;
  for (int i=0; i<3; i++) {
    go();  
  }
  if (score > TOP) {
    win();
  } else {
    lost();
  }

  // for demo play
  played = true;
  nextPlay = millis() + 35000;
}

void demo() {
  score = 0;  
  for (int i=0; i<3; i++) {
    int r = random(-4, 10);
//    Serial.print("r=");
//    Serial.println(r);
    go(r);  
  }
  if (score > TOP) {
    win();
  } else {
    lost();
  }
  nextPlay = millis() + 35000;
}


void setup() {
  Serial.begin(115200);
  randomSeed(analogRead(0));
  pinMode(buttonPin, INPUT_PULLUP); // input with pull-up resistor

  rgb.begin();
  rgb.show(); // Initialize all pixels to 'off'
  
  // for demo play
  played = false;


}

void loop() {
  while (!buttonPressed()) {
    theaterChaseRainbow();
    if (played && (millis() > nextPlay)) {
      demo();
    }
  }
  play();
}


void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<rgb.numPixels(); i++) {
      rgb.setPixelColor(i, Wheel((i+j) & 255));
    }
    rgb.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< rgb.numPixels(); i++) {
      rgb.setPixelColor(i, Wheel(((i * 256 / rgb.numPixels()) + j) & 255));
    }
    rgb.show();
    delay(wait);
  }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow() {
  for (int j=0; j<256; j=j+4) {     // cycle all 256 colors in the wheel
    for (int q=0; q<3; q++) {
      for (int i=0; i<rgb.numPixels(); i=i+3) {
        rgb.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every third pixel on
      }
      rgb.show();
      if (buttonPressed()) return;

      for (int i=0; i < rgb.numPixels(); i=i+3) {
        rgb.setPixelColor(i+q, 0);        //turn every third pixel off
      }
      delay(20);
    }
    delay(40);
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return rgb.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return rgb.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return rgb.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
