// Sample program to control the RGB LED ring with WS2812 controller
// Connect DI to pin 8
// 5V to 5V
// GND to GND
// Change the LED_COUNT to match yours

#include <WS2812.h>

#define DI_PIN 8     // Digital pin that the DI signal is connected to
#define LED_COUNT 8  // Number of LEDs in the ring or strip

WS2812 leds = WS2812(LED_COUNT, DI_PIN, NEO_GRB + NEO_KHZ800);

// Input a value 0 to 255 to get a color value.
// The colors are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  if(WheelPos < 85) {
   return leds.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  } else if(WheelPos < 170) {
   WheelPos -= 85;
   return leds.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else {
   WheelPos -= 170;
   return leds.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
}

// turn on one led with given color
// the parameters to setPixelColor(lednumber, red, green, blue)
// the colors red, green and blue is a number between 0 and 255
void oneLed() {
  leds.setPixelColor(1, 170, 0, 0);  // Set just this one
  leds.setPixelColor(2, 0, 200, 0);  // Set just this one
  leds.setPixelColor(6, 0, 0, 240);  // Set just this one
  leds.show();
  delay(8000);
  leds.clear();
}

// Rainbow lights
void rainbow(int times, int wait) {
  for(int j=256*times-1; j>=0; j--) { // 5 cycles of all colors on wheel
    for(int i=0; i<LED_COUNT; i++) {  // counterclockwise
      leds.setPixelColor(i, Wheel(((i * 256 / LED_COUNT) + j) & 255));
    }
    leds.show();
    delay(wait);
  }
  leds.clear();
}


//Theatre-style crawling lights
void theaterChase() {
  uint32_t color;
  for (int t=0; t<70; t++) {  //do times cycles of chasing
    color = Wheel(random(0, 255));
    for (int q=0; q<3; q++) {
      for (int i=0; i<leds.numPixels(); i=i+3) {
        leds.setPixelColor(i+q, color);    //turn every third pixel on
      }
      leds.show();
      delay(60);

      for (int i=0; i<leds.numPixels(); i=i+3) {
        leds.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
  leds.clear();
}

// Fill the dots one after the other
void wipe() {
  uint32_t color;
  for (int t=0; t<10; t++) {
    color = Wheel(random(0, 255));
    // up direction
    for (int i=0; i<leds.numPixels(); i++) {
      leds.setPixelColor(i, color);
      leds.show();
      delay(80);
    }
    // down direction
    for (int i=leds.numPixels()-1; i>=0; i--) {
      leds.setPixelColor(i, 0); // turn off
      leds.show();
      delay(80);
    }
  }
  leds.clear();
}

void setup() {
  leds.begin();      // Call this to start up the LED strip.
  leds.clear();      // turns all LEDs off...
  leds.setBrightness(255);  // 0=off; 255=full bright
}

void loop() {
  oneLed();
  rainbow(5, 10);
  theaterChase();
  wipe();
}
