// Sample program to control the RGB light ring/string with WS2812 controller
// using the various light patterns defined in WS2812patterns.h
// and the user-defined non-blocking delay callback function

//#define NONBLOCKING_DELAY
// Do delay without using the delay() function
// This is the user-defined non-blocking delay callback function used by all the routines in WS2812patterns.h
// must put this BEFORE the #include <WS2812patterns.h>
void nonblockingDelay_WS2812(int wait) {
  unsigned long timesUp;
  timesUp = millis() + wait;
  while (millis() < timesUp) {
    //server.handleClient();
    yield();  
  }
}

#define DI_PIN 8       // Digital pin that the DI signal is connected to
#define LED_COUNT 8  // Number of LEDs in the ring or strip
#include <WS2812.h>
WS2812 leds = WS2812(LED_COUNT, DI_PIN, NEO_GRB + NEO_KHZ800);
// Need to define the NONBLOCKING_DELAY and function above 
// BEFORE the following #include <WS2812patterns.h>
// because the nonblockingDelay_WS2812() callback function
// is used by all the routines in WS2812patterns.h
#include <WS2812patterns.h>


void setup() {
  Serial.begin(115200);
  randomSeed(analogRead(0));
  leds.begin();      // Call this to start up the LED strip.
  leds.clear();      // turns all LEDs off...
  leds.setBrightness(100);  // 0=off; 255=full bright

//  solid(leds, nonblockingDelay_WS2812, Color(0,0,100), 1,5, 5000);
//  solid(leds, nonblockingDelay_WS2812, 0, 1,5, 5000);
//  allOn(leds, nonblockingDelay_WS2812, Color(0,0,100), 5000);
//  allOn(leds, nonblockingDelay_WS2812, 0, 5000);
//  wipe(leds, nonblockingDelay_WS2812, Color(0,100,100), 0,0,2);
//  rainbow(leds, nonblockingDelay_WS2812);
//  movingBlocks(leds, nonblockingDelay_WS2812);
//  theaterChase(leds, nonblockingDelay_WS2812);
//  theaterChaseRainbow(leds, nonblockingDelay_WS2812);
//  randomFlash(leds, nonblockingDelay_WS2812);
//  discoLights(leds, nonblockingDelay_WS2812);
//  moveIn(leds, nonblockingDelay_WS2812);
//  fade(leds, nonblockingDelay_WS2812);
//  fadingRainbow(leds, nonblockingDelay_WS2812);
//  cyclone(leds, nonblockingDelay_WS2812);
//  flash(leds, nonblockingDelay_WS2812);
//  single(leds, nonblockingDelay_WS2812, 0, 1000, 0);
  allOff(leds);
}

void loop() {
  allOn(leds, nonblockingDelay_WS2812, selectColor(random(0,7)), 5000);
  wipe(leds, nonblockingDelay_WS2812, 0,0,0,2);
  rainbow(leds, nonblockingDelay_WS2812);
  movingBlocks(leds, nonblockingDelay_WS2812);
  theaterChaseRainbow(leds, nonblockingDelay_WS2812); 
  randomFlash(leds, nonblockingDelay_WS2812);
  discoLights(leds, nonblockingDelay_WS2812);
  moveIn(leds, nonblockingDelay_WS2812);
  fade(leds, nonblockingDelay_WS2812);
  fadingRainbow(leds, nonblockingDelay_WS2812);
  cyclone(leds, nonblockingDelay_WS2812);
  flash(leds, nonblockingDelay_WS2812);
  single(leds, nonblockingDelay_WS2812);
}
